function [misfit] = Hyp_misfit(xap,xvarn,N,Fval)

% **************************
%
% calculates misfit from a set of points to a hyperbolic curve or surface
% for binary/ternary ratio-ratio mixing problem
%
% then sums these distances to get a misfit
% also calculates misfit weighted by prior variance
%
% RAS, 2/11
%
% **************************

% set convergence parameters
fmin=1.e-4;
itermax=10000;
tinc0=-5.e-3;

Xn=xap(1:N);
Yn=xap(N+1:2*N);
xw=sqrt(xvarn(1:N));
yw=sqrt(xvarn(N+1:2*N));
misfit=zeros(N,2);

if Fval==4	% binary case
  X10n=xap(2*N+1);
  X20n=xap(2*N+2);  
  Y10n=xap(2*N+3);
  Y20n=xap(2*N+4);  
  K0=xap(2*N+5);

% calculate coefficients
  A=Y20n-Y10n*K0;
  B=K0-1;
  C=X10n-X20n*K0;
  D=Y10n*X20n*K0-Y20n*X10n;
  a1=1;
  b1=1;
  a2=1;
  b2=(K0*a2*b1)/a1;

else		% ternary case
% fill variables
  Zn=xap(2*N+1:3*N);
  zw=sqrt(xvarn(2*N+1:3*N));
  X10n=xap(3*N+1);
  X20n=xap(3*N+2);
  X30n=xap(3*N+3);
  Y10n=xap(3*N+4);
  Y20n=xap(3*N+5);
  Y30n=xap(3*N+6);
  Z10n=xap(3*N+7);
  Z20n=xap(3*N+8);
  Z30n=xap(3*N+9);
  Ka2b10=xap(3*N+10);
  Kb1c20=xap(3*N+11);
  Ka1c30=xap(3*N+12);
  Ka1b30=xap(3*N+13);

% calculate coefficients
  A=Y20n*Z10n-Kb1c20*Y10n*Z20n+Ka2b10*Ka1c30*Y10n*Z30n-Ka1b30*Ka2b10*Y30n*Z10n+Ka1b30*Kb1c20*Y30n*Z20n-Ka1c30*Y20n*Z30n;
  B=X30n*Z10n-Ka1c30*X10n*Z30n+Ka1b30*Kb1c20*X10n*Z20n-Ka1b30*Ka2b10*X20n*Z10n+Ka2b10*Ka1c30*X20n*Z30n-Kb1c20*X30n*Z20n;
  C=X30n*Y20n-Ka1b30*Ka2b10*X20n*Y30n+Ka2b10*Ka1c30*X20n*Y10n-Ka1c30*X10n*Y20n+Ka1b30*Kb1c20*X10n*Y30n-Kb1c20*X30n*Y10n;
  D=Ka1c30*(1-Ka2b10)*Z30n+Kb1c20*(1-Ka1b30)*Z20n+(Ka1b30*Ka2b10-1)*Z10n;
  E=Ka1b30*(Ka2b10-Kb1c20)*Y30n+(Ka1c30-1)*Y20n+(Kb1c20-Ka2b10*Ka1c30)*Y10n;
  F=(Kb1c20-1)*X30n+Ka2b10*(Ka1b30-Ka1c30)*X20n+(Ka1c30-Ka1b30*Kb1c20)*X10n;
  G=1-Ka1b30*Ka2b10+Kb1c20*(Ka1b30-1)+Ka1c30*(Ka2b10-1);
  H=Ka1c30*X10n*Y20n*Z30n-X30n*Y20n*Z10n+Ka1b30*Ka2b10*X20n*Y30n*Z10n-Ka2b10*Ka1c30*X20n*Y10n*Z30n+Kb1c20*X30n*Y10n*Z20n-Ka1b30*Kb1c20*X10n*Y30n*Z20n;

% set concentration absolute values for plotting surface
  a1=1;               % set reference values
  b1=1;               % have 4 contraints and 9 variables, so 5 ind. vars must be set
  c1=1;
  a2=1;
  a3=1;
  b2=(a2*b1)/(Ka2b10*a1);
  c2=(Kb1c20*c1*a2)/(Ka2b10*a1);
  b3=(Ka1b30*a3*b1)/a1;
  c3=(Ka1c30*a3*c1)/a1;

end

% step through points and find distance to surface
  for j=1:N
    flag=0;	% convergence flag

% get initial misfit for f
    Z(1)=Xn(j);
    Z(2)=Yn(j);
    
    if Fval==4
      Z(3:7)=xap(2*N+1:2*N+5);
      f0=getf(Z,1,7,4);
    else
      Z(3)=Zn(j);
      Z(4:16)=xap(3*N+1:3*N+13);
      f0=getf(Z,1,16,7);
    end

    tinc=tinc0;

% initialize iterative solver
    x0=Xn(j);
    y0=Yn(j);
    x1=x0;
    y1=y0;
    count=0;
    dx=0;
    dy=0;
    f1=f0;
    if Fval==7
      z0=Zn(j);
      z1=z0;
      dz=0;
    end

% plot progression to see how its working
    if Fval==4
      xd0=(X10n-X20n)/50;
      xs0=X20n:xd0:X10n;
      ys0=-(A*xs0+D)./(B*xs0+C);
    else
      zs0n=ones(20,21)*NaN;
      ys0n=ones(20,21)*NaN;
      xs0n=ones(20,21)*NaN;

      cnt=0;
      cx=0;
      for F1=0:0.05:1;
        cx=cx+1;
        cy=0;
        df2=(1-F1)/20;
        for F2=0:df2:1-F1;
          cy=cy+1;
          F3=1-F1-F2;
          cnt=cnt+1;
          xs0n(cx,cy)=(F1*a1*X10n+F2*a2*X20n+F3*a3*X30n)/(F1*a1+F2*a2+F3*a3);
          ys0n(cx,cy)=(F1*b1*Y10n+F2*b2*Y20n+F3*b3*Y30n)/(F1*b1+F2*b2+F3*b3);
          zs0n(cx,cy)=(F1*c1*Z10n+F2*c2*Z20n+F3*c3*Z30n)/(F1*c1+F2*c2+F3*c3);
        end
      end
    end
%   figure(6)
%   clf
   if Fval==4
%     plot(xs0,ys0,'r-')
%     hold on
%     plot(X10n,Y10n,'ro')
%     plot(X20n,Y20n,'ro')
   else 
%    pS0=surf(xs0n,ys0n,zs0n);
%    set(pS0,'EdgeColor','r');
%    set(pS0,'FaceColor','yellow');
%    set(pS0,'FaceAlpha',[0.8]);
%    hold on
%    plot3(Z(1),Z(2),Z(3),'k*');
   end
    
    while flag==0
      count=count+1;
      if Fval==4
        gx=A+B*y1;
        gy=B*x1+C;
      else
        gx=A+D*y1+E*z1+G*y1*z1;
        gy=B+D*x1+F*z1+G*x1*z1;
        gz=C+E*x1+F*y1+G*x1*y1;
      end
      dx=dx+gx*tinc;
      dy=dy+gy*tinc;
      x2=x0+dx;
      y2=y0+dy;
      Z(1)=x2;
      Z(2)=y2;
      if Fval==4
        f2=getf(Z,1,7,4);
%        plot(Z(1),Z(2),'b*');
      else
        dz=dz+gz*tinc;
        z2=z0+dz;
        Z(3)=z2;
        f2=getf(Z,1,16,7);
%       plot3(Z(1),Z(2),Z(3),'b*');
      end

%      pause

      if (sign(f2) ~= sign(f1))	% have crossed to other side of surface
        dx=dx-gx*tinc;
        dy=dy-gy*tinc;
        if Fval==7
          dz=dz-gz*tinc;
        end
        tinc=tinc*.5;
      elseif (abs(f2) >= abs(f1) & count==1)
        dx=dx-gx*tinc;
        dy=dy-gy*tinc;
        if Fval==7
          dz=dz-gz*tinc;
        end
        tinc=-tinc;
      elseif abs(f2) < fmin
        flag=1;
      elseif count > itermax
        fprintf('\nHyp_misfit did not converge after %d iterations for data point # %d',itermax,j);
        fprintf('\nObjective function, f = %f, greater than convergence criterion %f',f2,fmin);
        fprintf('\nMisfit estimate may be in error\n\n');
        flag=1;
      else
        x1=x2;
        y1=y2;
        if Fval==7
          z1=z2;
        end
        f1=f2;
      end

    end		% end while
 
    if Fval==4
      misfit(j,1)=sqrt((x0-x2)^2+(y0-y2)^2);
      misfit(j,2)=sqrt(((x0-x2)/xw(j))^2+((y0-y2)/yw(j))^2);
    else
      misfit(j,1)=sqrt((x0-x2)^2+(y0-y2)^2+(z0-z2)^2);
      misfit(j,2)=sqrt(((x0-x2)/xw(j))^2+((y0-y2)/yw(j))^2+((z0-z2)/zw(j))^2);
    end

  end
      
end
