function [f] = getf(Z,N,M,Fval)

%
% **********************************
%
% function calculator
%
% orignal code, RAS 8/01
% mods made with addition of each new Fval
% settling velocity, 8/10
% ternary mixing, RAS 1/11
%
% **********************************

% calculates function value for non-linear maximum likelihood
% routine MLEiter
%
% inputs are;
% (1) Z, vector with all data required to calculate F
% (2) N, #rows in F
% (3) M, #columns in F
% (4) Fval, an index describing the nature of the governing equation
%

% Fval=1 is a straight line equation
% Fval=2,3 is the crysal-melt partition coefficient equation
% Fval=4 is binary mixing
% Fval=5 is two-step diffusion
% Fval=6 is Ferguson and Church settling velocity equation
% Fval=7 is ternary mixing

if Fval==1

  f=zeros(N,1);
  for j=1:N
     f(j)=Z(2*N+1)*Z(j)+Z(2*N+2)-Z(N+j);
  end
 

elseif Fval==2

  f=zeros(N,1);
  Na=6.022e23;	% Avogadro's #
  R=8.314;	% Gas constant, g-moles-joules
  T=Z(2*N+4);	% temp for fit
  C=-4*pi*Na/(R*T);	% constant term
  Di=Z(1:N);
  ri=Z(N+4:2*N+3);
  r0=Z(N+3);
  E=Z(N+2);
  D0=Z(N+1);
  dr=ri-r0;
  A=exp(C*E*((r0/2)*dr.^2+(dr.^3)/3));

  f=D0*A-Di;

elseif Fval==3

  f=zeros(N,1);

  Na=6.022e23;  % Avogadro's #
  R=8.314;      % Gas constant, g-moles-joules

  T=Z(2*N+4);   % temp for fit
  C=-4*pi*Na/(R*T);     % constant term
  Di=Z(1:N);
  ri=Z(N+4:2*N+3);
  r0=Z(N+3);
  E=Z(N+2);
  D0=Z(N+1);
  dr=ri-r0;
  A=10^(-20)*C*E*((r0/2)*dr.^2+(dr.^3)/3);

  f=log(D0)+A-log(Di);

elseif Fval==4

  f=zeros(N,1);

  RXA=Z(2*N+1);
  RXB=Z(2*N+2);
  RYA=Z(2*N+3);
  RYB=Z(2*N+4);
  K=Z(2*N+5);
  RXM=Z(1:N);
  RYM=Z(N+1:2*N);

  a=RYB-RYA*K;
  b=K-1;
  c=RXA-RXB*K;
  d=RYA*RXB*K-RYB*RXA;

  f=a*RXM+b*RXM.*RYM+c*RYM+d;

elseif Fval==5

  f=zeros(N,1);

  DA=Z(N+1);
  DB=Z(N+2);
  eA=Z(N+3);
  eB=Z(N+4);
  Temp=Z(N+5:2*N+4);
  R=Z(2*N+5);

  f=DA*exp(eA./(R*Temp))+DB*exp(eB./(R*Temp))-Z(1:N);

elseif Fval==6

  f=zeros(N,1);

  g=9.81;	% acceleration of gravity, m/s^2
  nu=1.83e-06;	% kinematic viscosity of seawater, m^2/s
  R=2.61;	% immersed specific gravity of clasts, dimensionless

  D=Z(1:N);	% grain size, m
  w=Z(N+1:2*N);	% settling velocities, m/s
  C1=Z(2*N+1);	% equation coefficients
  C2=Z(2*N+2);

  f=ones(N,1)*C1*nu+sqrt(C2*3/4*R*g*D.^3)-R*g*D.^2./w;

elseif Fval==7

  f=zeros(N,1);

  X1=Z(3*N+1);
  X2=Z(3*N+2);
  X3=Z(3*N+3);
  Y1=Z(3*N+4);
  Y2=Z(3*N+5);
  Y3=Z(3*N+6);
  Z1=Z(3*N+7);
  Z2=Z(3*N+8);
  Z3=Z(3*N+9);
  Ka2b1=Z(3*N+10);
  Kb1c2=Z(3*N+11);
  Ka1c3=Z(3*N+12);
  Ka1b3=Z(3*N+13);
  x=Z(1:N);
  y=Z(N+1:2*N);
  z=Z(2*N+1:3*N);

  A=Y2*Z1-Kb1c2*Y1*Z2+Ka2b1*Ka1c3*Y1*Z3-Ka1b3*Ka2b1*Y3*Z1+Ka1b3*Kb1c2*Y3*Z2-Ka1c3*Y2*Z3;
  B=X3*Z1-Ka1c3*X1*Z3+Ka1b3*Kb1c2*X1*Z2-Ka1b3*Ka2b1*X2*Z1+Ka2b1*Ka1c3*X2*Z3-Kb1c2*X3*Z2;
  C=X3*Y2-Ka1b3*Ka2b1*X2*Y3+Ka2b1*Ka1c3*X2*Y1-Ka1c3*X1*Y2+Ka1b3*Kb1c2*X1*Y3-Kb1c2*X3*Y1;
  D=Ka1c3*(1-Ka2b1)*Z3+Kb1c2*(1-Ka1b3)*Z2+(Ka1b3*Ka2b1-1)*Z1;
  E=Ka1b3*(Ka2b1-Kb1c2)*Y3+(Ka1c3-1)*Y2+(Kb1c2-Ka2b1*Ka1c3)*Y1;
  F=(Kb1c2-1)*X3+Ka2b1*(Ka1b3-Ka1c3)*X2+(Ka1c3-Ka1b3*Kb1c2)*X1;
  G=1-Ka1b3*Ka2b1+Kb1c2*(Ka1b3-1)+Ka1c3*(Ka2b1-1);
  H=Ka1c3*X1*Y2*Z3-X3*Y2*Z1+Ka1b3*Ka2b1*X2*Y3*Z1-Ka2b1*Ka1c3*X2*Y1*Z3+Kb1c2*X3*Y1*Z2-Ka1b3*Kb1c2*X1*Y3*Z2;

  f=A*x+B*y+C*z+D*x.*y+E*x.*z+F*y.*z+G*x.*y.*z+H;

else

  error('Unrecognized Fval')

end
